import { RequestData } from './definitions';
import { genID } from './helpers';

export class RDC {
  private port: MessagePort;
  private pending: {
    [id: string]: {
      resolve: Function,
      reject: Function
    }
  } = {};

  constructor(port: MessagePort) {
    this.port = port;

    this.port.onmessage = (e) => {
      const reqid = e.data.payload.__reqid;
      
      if (this.pending[reqid]) {
        delete e.data.payload.__reqid;
        // Null the payload if empty object
        const res = (Object.keys(e.data.payload).length === 0 && e.data.payload.constructor === Object) ? null : e.data.payload;
        this.pending[reqid].resolve(res);
        delete this.pending[reqid];
      }
    }
  }

  // Always returns a promise; uniquely ID's messages being sent.
  request(data: RequestData) {
    // Generate request ID
    const id = genID();
    return new Promise((resolve, reject) => {
      this.pending[id] = {resolve, reject};

      // Ensure the payload object includes the request ID
      data.payload.__reqid = id;

      this.port.postMessage(data);
    });
  }
}
